@@ -27,8 +27,11 @@
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
  *   nmap-os-fingerprints or nmap-service-probes.                          *
- * o Executes Nmap                                                         *
- * o Integrates/includes/aggregates Nmap into an executable installer      *
+ * o Executes Nmap and parses the results (as opposed to typical shell or  *
+ *   execution-menu apps, which simply display raw Nmap output and so are  *
+ *   not derivative works.)                                                * 
+ * o Integrates/includes/aggregates Nmap into a proprietary executable     *
+ *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
@@ -55,8 +58,17 @@
  * the continued development of Nmap technology.  Please email             *
  * sales@insecure.com for further information.                             *
  *                                                                         *
+ * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * combinations including the two. You must obey the GNU GPL in all        *
+ * respects for all of the code used other than OpenSSL.  If you modify    *
+ * this file, you may extend this exception to your version of the file,   *
+ * but you are not obligated to do so.                                     *
+ *                                                                         *
  * If you received these files with a written license agreement or         *
- * contract stating terms other than the (GPL) terms above, then that      *
+ * contract stating terms other than the terms above, then that            *
  * alternative license agreement takes precedence over these comments.     *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
@@ -82,11 +94,12 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html .                                  *
+ * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
+ * with Nmap.                                                              *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nmap_rpc.cc,v 1.15 2004/03/12 01:59:04 fyodor Exp $ */
+/* $Id: nmap_rpc.cc,v 1.20 2004/08/29 09:12:03 fyodor Exp $ */
 
 
 #include "nmap_rpc.h"
@@ -203,7 +216,7 @@ int send_rpc_query(const struct in_addr *target_host, unsigned short portno,
   struct sockaddr_in sock;
   char rpch_buf[256]; 
   struct rpc_hdr *rpch;
-  int res;
+  int res, err = 0;
 
   /* static int numruns = 0;
      if (numruns++ > 2)
@@ -289,7 +302,11 @@ int send_rpc_query(const struct in_addr *target_host, unsigned short portno,
 	hdump((unsigned char *) rpch, sizeof(struct rpc_hdr));
       res = sendto(udp_rpc_socket, (char *)rpch, sizeof(struct rpc_hdr), 0,
 		   (struct sockaddr *) &sock, sizeof(struct sockaddr_in));
-    } while(res == -1 && (socket_errno() == EINTR || socket_errno() == ENOBUFS));
+      if (res == -1)
+	err = socket_errno();
+     } while(res == -1 && (err == EINTR || err == ENOBUFS));
+
+
     if (res == -1) {
       if (o.debugging) {
 	gh_perror("Sendto in send_rpc_query");
@@ -376,8 +393,8 @@ int rpc_are_we_done(char *msg, int msg_len, Target *target,
     
    
   if (current->state != PORT_TESTING && current->state != PORT_CLOSED &&
-      current->state != PORT_FIREWALLED) {
-    error("Supposed scan_offset refers to port in state %s (should be testing,closed, or firewalld)", statenum2str(current->state));
+      current->state != PORT_FILTERED) {
+    error("Supposed scan_offset refers to port in state %s (should be testing,closed, or filtered)", statenum2str(current->state));
     rsi->rpc_status = RPC_STATUS_NOT_RPC;
     ss->numqueries_outstanding = 0;
     return 1;
@@ -493,13 +510,9 @@ unsigned long current_msg_len;
 
    
    /* Insure there is no timeout ... */
-   if (o.host_timeout) {	
-     gettimeofday(&tv, NULL);
-     if (TIMEVAL_MSEC_SUBTRACT(tv, target->host_timeout) >= 0) {
-       target->timedout = 1;
+   gettimeofday(&tv, NULL);
+   if (target->timedOut(&tv))
        return;
-     }
-   }
 
    tv.tv_sec = target->to.timeout / 1000000;
    tv.tv_usec = target->to.timeout % 1000000;
